import * as d3 from 'd3';

function renderData(dataElements) {

    let width = 600;
    let height = 400;

    let years = new Set(dataElements.map(o => +o.Year));
    let states = new Set(dataElements.map(o => o.State));
    let rates = dataElements.map(o => +o.Abortion_Rate);

    let data = dataElements.map(o => ({
        year: +o.Year,
        state: o.State,
        rate: +o.Abortion_Rate
    }));

    // nested function that groups data by state
    let dataGrouped = d3.group(data, d => d.state);

    let margin = ({ top: 40, right: 20, bottom: 35, left: 40, legend: 50 })

    let x = d3.scaleLinear()
        .domain(d3.extent(years)).nice()
        .range([margin.left, width - margin.right]);

    let y = d3.scaleLinear()
        .domain(d3.extent(rates)).nice()
        .range([height - margin.bottom, margin.top]);

    // function with g arg assigned to xAxis
    let xAxis = g => g
        .attr("transform", `translate(0,${height - margin.bottom})`)
        .call(d3.axisBottom(x).tickFormat(d3.format("d")));

    // function with g arg assigned to yAxis
    let yAxis = g => g
        .attr("transform", `translate(${margin.left},0)`)
        .call(d3.axisLeft(y));

    // colors for the states
    // https://colorbrewer2.org/?type=qualitative&scheme=Set1&n=8

    let brewer = colorbrewer.Set1;
    const svg = d3.select("body").append("svg")
        .attr("width", width + margin.left + margin.right)
        .attr("height", height + margin.top + margin.bottom);

    console.log(brewer[8]);
    const color = d3.scaleOrdinal()
        .domain(states)
        .range(brewer[8]);

    // legend
    let legend = d3.scaleOrdinal()
        .domain(states)
        .range([0, 20, 40, 60, 80, 100, 120, 140]);

    svg.append("g")
        .attr("transform", `translate(${width - 2*margin.right},${margin.legend})`)
        .call(d3.axisLeft(legend));

    svg.append("g").selectAll("rect")
        .data(brewer[8])
        .join("rect")
        .attr("fill", color)
        .attr("width", 16)
        .attr("height", 16)
        .attr("x", width - 2 * margin.right + 1)
        .attr("y", d => margin.legend - 8 + legend(d));

    svg.append("text")
        .attr("fill", "black")
        .attr("x", width - 3 * margin.right)
        .attr("y", "27")
        .style("font-size", "12px")
        .text("States Participated");

    svg.append("text")
        .attr("fill", "black")
        .attr("x", width - 3 * margin.right)
        .attr("y", "37")
        .style("font-size", "12px")
        .text("in the Research:");

    // div for the tooltip
    var tooltip = d3.select("body").append("div")
        .attr("class", "tooltip")
        .style("opacity", 0);

    svg.append("g")
        .call(xAxis);

    svg.append("g")
        .call(yAxis);

    // add lines
    svg.selectAll(".line")
        .data(dataGrouped)
        .join("path")
        .attr("fill", "none")
        .attr("stroke", function(d) {
            return color(d[0])
        })
        .attr("stroke-width", 2.5)
        .attr("d", function(d) {
            return d3.line()
                .x(function(d) { return x(d.year) })
                .y(function(d) { return y(d.rate) })
                (d[1])
        });

    svg.append("g")
        .selectAll("circle")
        .data(data)
        .join("circle")
        .attr("fill", "#292929")
        .attr("cx", function(d) { return x(d.year); })
        .attr("cy", function(d) { return y(d.rate); })
        .attr("r", 3)
        .on("mouseover", function(d, data) {
            tooltip.transition()
                .duration(200)
                .style("opacity", .85);

            tooltip.style("position", "absolute")
                .style("text-align", "center")
                .style("padding", "5px")
                .style("font-size", "12.5px")
                .style("background", "rgb(231, 231, 229)")
                .style("border", "0px")
                .style("border-radius", "6px")
                .style("pointer-events", "none");

            tooltip.html("In " + data.year + " in " + data.state + "<br/>" + "Abortion rate was " + data.rate + ".")
                .style("left", d3.select(this).attr("cx") + "px")
                .style("top", d3.select(this).attr("cy") + "px");
        })
        .on("mouseout", function(d) {
            tooltip.transition()
                .duration(500)
                .style("opacity", 0);
        });

    svg.append('text')
        .attr("fill", "black")
        .attr("x", width / 2.2)
        .attr("y", height - 2)
        .style("font-size", "14px")
        .text("Years");

    svg.append("text")
        .attr("fill", "black")
        .attr("x", "150")
        .attr("y", "-15")
        .attr("transform", "translate(30, 400) rotate(270)")
        .style("font-size", "14px")
        .text("Rates");

    svg.append("text")
        .attr("fill", "black")
        .attr("x", width / 5)
        .attr("y", "25")
        .style("font-size", "18px")
        .text("Abortion Rates for Group 15-17 Years Old");
}

d3.csv(
    "https://gist.githubusercontent.com/olga-kondr/f332e6fce0501c81d52054a83b704c05/raw/b4fadcb70215227ebd2fcaacc416b3da481e4d49/consequent_rates.csv"
).then(data => renderData(data));